@@ -1,4 +1,3 @@
-# Lint as: python3
 # Copyright 2019 DeepMind Technologies Limited. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -22,20 +21,48 @@ from optax._src.alias import adamw
 from optax._src.alias import dpsgd
 from optax._src.alias import fromage
 from optax._src.alias import lamb
+from optax._src.alias import lars
 from optax._src.alias import noisy_sgd
 from optax._src.alias import radam
 from optax._src.alias import rmsprop
 from optax._src.alias import sgd
+from optax._src.alias import sm3
 from optax._src.alias import yogi
+from optax._src.base import EmptyState
+from optax._src.base import GradientTransformation
+from optax._src.base import identity
+from optax._src.base import OptState
+from optax._src.base import Params
+from optax._src.base import Schedule
+from optax._src.base import TransformInitFn
+from optax._src.base import TransformUpdateFn
+from optax._src.base import Updates
+from optax._src.clipping import adaptive_grad_clip
+from optax._src.clipping import AdaptiveGradClipState
+from optax._src.clipping import clip
+from optax._src.clipping import clip_by_block_rms
+from optax._src.clipping import clip_by_global_norm
+from optax._src.clipping import ClipByGlobalNormState
+from optax._src.clipping import ClipState
 from optax._src.combine import chain
+from optax._src.constrain import keep_params_nonnegative
+from optax._src.constrain import NonNegativeParamsState
+from optax._src.constrain import zero_nans
+from optax._src.constrain import ZeroNansState
 from optax._src.control_variates import control_delta_method
 from optax._src.control_variates import control_variates_jacobians
 from optax._src.control_variates import moving_avg_baseline
+from optax._src.linear_algebra import global_norm
 from optax._src.linear_algebra import matrix_inverse_pth_root
 from optax._src.linear_algebra import power_iteration
+from optax._src.lookahead import lookahead
+from optax._src.lookahead import LookaheadParams
+from optax._src.lookahead import LookaheadState
 from optax._src.loss import cosine_distance
+from optax._src.loss import cosine_similarity
 from optax._src.loss import huber_loss
 from optax._src.loss import l2_loss
+from optax._src.loss import log_cosh
 from optax._src.loss import sigmoid_binary_cross_entropy
 from optax._src.loss import smooth_labels
 from optax._src.loss import softmax_cross_entropy
@@ -47,18 +74,21 @@ from optax._src.schedule import cosine_onecycle_schedule
 from optax._src.schedule import exponential_decay
 from optax._src.schedule import inject_hyperparams
 from optax._src.schedule import InjectHyperparamsState
+from optax._src.schedule import join_schedules
 from optax._src.schedule import linear_onecycle_schedule
+from optax._src.schedule import linear_schedule
 from optax._src.schedule import piecewise_constant_schedule
 from optax._src.schedule import piecewise_interpolate_schedule
 from optax._src.schedule import polynomial_schedule
-from optax._src.schedule import Schedule
+from optax._src.schedule import sgdr_schedule
+from optax._src.schedule import warmup_cosine_decay_schedule
+from optax._src.schedule import warmup_exponential_decay_schedule
 from optax._src.second_order import fisher_diag
 from optax._src.second_order import hessian_diag
 from optax._src.second_order import hvp
 from optax._src.stochastic_gradient_estimators import measure_valued_jacobians
 from optax._src.stochastic_gradient_estimators import pathwise_jacobians
 from optax._src.stochastic_gradient_estimators import score_function_jacobians
-from optax._src.transform import adaptive_grad_clip
 from optax._src.transform import add_decayed_weights
 from optax._src.transform import add_noise
 from optax._src.transform import AddDecayedWeightsState
@@ -68,24 +98,18 @@ from optax._src.transform import AddNoiseState
 from optax._src.transform import apply_every
 from optax._src.transform import ApplyEvery
 from optax._src.transform import centralize
-from optax._src.transform import clip
-from optax._src.transform import clip_by_global_norm
-from optax._src.transform import ClipByGlobalNormState
-from optax._src.transform import ClipState
-from optax._src.transform import global_norm
-from optax._src.transform import GradientTransformation
-from optax._src.transform import identity
-from optax._src.transform import keep_params_nonnegative
-from optax._src.transform import NonNegativeParamsState
-from optax._src.transform import OptState
-from optax._src.transform import Params
+from optax._src.transform import ema
+from optax._src.transform import EmaState
 from optax._src.transform import scale
 from optax._src.transform import scale_by_adam
 from optax._src.transform import scale_by_belief
+from optax._src.transform import scale_by_param_block_norm
+from optax._src.transform import scale_by_param_block_rms
 from optax._src.transform import scale_by_radam
 from optax._src.transform import scale_by_rms
 from optax._src.transform import scale_by_rss
 from optax._src.transform import scale_by_schedule
+from optax._src.transform import scale_by_sm3
 from optax._src.transform import scale_by_stddev
 from optax._src.transform import scale_by_trust_ratio
 from optax._src.transform import scale_by_yogi
@@ -95,15 +119,11 @@ from optax._src.transform import ScaleByRmsState
 from optax._src.transform import ScaleByRssState
 from optax._src.transform import ScaleByRStdDevState
 from optax._src.transform import ScaleByScheduleState
+from optax._src.transform import ScaleBySM3State
 from optax._src.transform import ScaleByTrustRatioState
 from optax._src.transform import ScaleState
 from optax._src.transform import trace
 from optax._src.transform import TraceState
-from optax._src.transform import TransformInitFn
-from optax._src.transform import TransformUpdateFn
-from optax._src.transform import Updates
-from optax._src.transform import zero_nans
-from optax._src.transform import ZeroNansState
 from optax._src.update import apply_updates
 from optax._src.update import incremental_update
 from optax._src.update import periodic_update
@@ -111,9 +131,6 @@ from optax._src.utils import multi_normal
 from optax._src.wrappers import apply_if_finite
 from optax._src.wrappers import ApplyIfFiniteState
 from optax._src.wrappers import flatten
-from optax._src.wrappers import lookahead
-from optax._src.wrappers import LookaheadParams
-from optax._src.wrappers import LookaheadState
 from optax._src.wrappers import masked
 from optax._src.wrappers import MaskedState
 from optax._src.wrappers import maybe_update
@@ -121,13 +138,14 @@ from optax._src.wrappers import MaybeUpdateState
 from optax._src.wrappers import MultiSteps
 from optax._src.wrappers import MultiStepsState
 
-__version__ = "0.0.6"
+__version__ = "0.0.8"
 
 __all__ = (
     "adabelief",
     "adagrad",
     "adam",
     "adamw",
+    "AdaptiveGradClipState",
     "adaptive_grad_clip",
     "add_decayed_weights",
     "add_noise",
@@ -143,6 +161,7 @@ __all__ = (
     "centralize",
     "chain",
     "clip",
+    "clip_by_block_rms",
     "clip_by_global_norm",
     "ClipByGlobalNormState",
     "ClipState",
@@ -152,9 +171,13 @@ __all__ = (
     "cosine_decay_schedule",
     "cosine_distance",
     "cosine_onecycle_schedule",
+    "cosine_similarity",
     "dpsgd",
     "differentially_private_aggregate",
     "DifferentiallyPrivateAggregateState",
+    "ema",
+    "EmaState",
+    "EmptyState",
     "exponential_decay",
     "fisher_diag",
     "flatten",
@@ -168,12 +191,16 @@ __all__ = (
     "incremental_update",
     "inject_hyperparams",
     "InjectHyperparamsState",
+    "join_schedules",
     "lamb",
+    "lars",
+    "log_cosh",
     "lookahead",
     "LookaheadParams",
     "LookaheadState",
     "l2_loss",
     "linear_onecycle_schedule",
+    "linear_schedule",
     "matrix_inverse_pth_root",
     "masked",
     "MaskedState",
@@ -194,10 +221,13 @@ __all__ = (
     "scale",
     "scale_by_adam",
     "scale_by_belief",
+    "scale_by_param_block_norm",
+    "scale_by_param_block_rms",
     "scale_by_radam",
     "scale_by_rms",
     "scale_by_rss",
     "scale_by_schedule",
+    "scale_by_sm3",
     "scale_by_stddev",
     "scale_by_trust_ratio",
     "scale_by_yogi",
@@ -207,11 +237,14 @@ __all__ = (
     "ScaleByRssState",
     "ScaleByRStdDevState",
     "ScaleByScheduleState",
+    "ScaleBySM3State",
     "ScaleByTrustRatioState",
     "ScaleState",
     "Schedule",
     "score_function_jacobians",
     "sgd",
+    "sgdr_schedule",
+    "sm3",
     "sigmoid_binary_cross_entropy",
     "smooth_labels",
     "softmax_cross_entropy",
@@ -220,6 +253,8 @@ __all__ = (
     "TransformInitFn",
     "TransformUpdateFn",
     "Updates",
+    "warmup_cosine_decay_schedule",
+    "warmup_exponential_decay_schedule",
     "yogi",
 )
 
