@@ -1,4 +1,3 @@
-# Lint as: python3
 # Copyright 2019 DeepMind Technologies Limited. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,18 +14,19 @@
 # ==============================================================================
 """Aliases for popular optimisers."""
 
-from typing import Union, Optional
+from typing import Any, Callable, Optional, Union
 
 import jax.numpy as jnp
 
+from optax._src import base
 from optax._src import combine
 from optax._src import privacy
-from optax._src import schedule
 from optax._src import transform
+from optax._src import wrappers
 
 
-GradientTransformation = transform.GradientTransformation
-ScalarOrSchedule = Union[float, schedule.Schedule]
+ScalarOrSchedule = Union[float, base.Schedule]
+MaskOrFn = Optional[Union[Any, Callable[[base.Params], Any]]]
 
 
 def _scale_by_learning_rate(learning_rate: ScalarOrSchedule):
@@ -39,7 +39,29 @@ def adabelief(
     learning_rate: ScalarOrSchedule,
     b1: float = 0.9,
     b2: float = 0.999,
-    eps: float = 1e-8) -> GradientTransformation:
+    eps: float = 1e-8) -> base.GradientTransformation:
+  """The AdaBelief optimiser.
+
+  AdaBelief is an adaptive learning rate optimiser that focuses on fast
+  convergence, generalisation, and stability. It adapts the step size depending
+  on its "belief" in the gradient direction — the optimiser adaptively scales
+  the step size by the difference between the predicted and observed gradients.
+  AdaBelief is a modified version of Adam and contains the same number of
+  parameters.
+
+  References:
+    Zhuang et al, 2020: https://arxiv.org/abs/2010.07468
+
+  Args:
+    learning_rate: this is a fixed global scaling factor.
+    b1: the exponential decay rate to track the first moment of past gradients.
+    b2: the exponential decay rate to track the second moment of past gradients.
+    eps: a small constant applied to denominator outside of the square root
+      (as in the Adam paper) to avoid dividing by zero when rescaling.
+
+  Returns:
+    the corresponding `GradientTransformation`.
+  """
   return combine.chain(
       transform.scale_by_belief(b1=b1, b2=b2, eps=eps),
       _scale_by_learning_rate(learning_rate),
@@ -49,7 +71,8 @@ def adabelief(
 def adagrad(
     learning_rate: ScalarOrSchedule,
     initial_accumulator_value: float = 0.1,
-    eps: float = 1e-7) -> GradientTransformation:
+    eps: float = 1e-7
+) -> base.GradientTransformation:
   """The Adagrad optimizer.
 
   Adagrad is an algorithm for gradient based optimisation that anneals the
@@ -60,13 +83,13 @@ def adagrad(
   during training and the learning rate eventually becomes vanishingly small.
 
   References:
-    [Duchi et al, 2011](https://jmlr.org/papers/v12/duchi11a.html)
+    Duchi et al, 2011: https://jmlr.org/papers/v12/duchi11a.html
 
   Args:
     learning_rate: this is a fixed global scaling factor.
     initial_accumulator_value: initialisation for the accumulator.
-    eps: a small constant applied to denominator outside of the square root
-      (as in the Adam paper) to avoid dividing by zero when rescaling.
+    eps: a small constant applied to denominator inside of the square root
+      (as in RMSProp) to avoid dividing by zero when rescaling.
 
   Returns:
     the corresponding `GradientTransformation`.
@@ -83,15 +106,16 @@ def adam(
     b1: float = 0.9,
     b2: float = 0.999,
     eps: float = 1e-8,
-    eps_root: float = 0.0) -> GradientTransformation:
+    eps_root: float = 0.0
+) -> base.GradientTransformation:
   """The classic Adam optimiser.
 
   Adam is an SGD variant with learning rate adaptation. The `learning_rate`
-  used for each weight is computed from estimates of first and second order
+  used for each weight is computed from estimates of first- and second-order
   moments of the gradients (using suitable exponential moving averages).
 
   References:
-    [Kingma et al, 2014](https://arxiv.org/abs/1412.6980)
+    Kingma et al, 2014: https://arxiv.org/abs/1412.6980
 
   Args:
     learning_rate: this is a fixed global scaling factor.
@@ -118,7 +142,9 @@ def adamw(
     b2: float = 0.999,
     eps: float = 1e-8,
     eps_root: float = 0.0,
-    weight_decay: float = 1e-4) -> GradientTransformation:
+    weight_decay: float = 1e-4,
+    mask: Optional[Union[Any, Callable[[base.Params], Any]]] = None,
+) -> base.GradientTransformation:
   """Adam with weight decay regularization.
 
   AdamW uses weight decay to regularise learning towards small weights, as
@@ -131,7 +157,7 @@ def adamw(
   variant where `additive_weight_decay` is applied only to a subset of `params`.
 
   References:
-    [Loshchilov et al, 2019](https://arxiv.org/abs/1711.05101)
+    Loshchilov et al, 2019: https://arxiv.org/abs/1711.05101
 
   Args:
     learning_rate: this is a fixed global scaling factor.
@@ -143,20 +169,45 @@ def adamw(
       square root (as in RMSProp), to avoid dividing by zero when rescaling.
       This is needed for instance when computing (meta-)gradients through Adam.
     weight_decay: strength of the weight decay regularization.
+    mask: a tree with same structure as (or a prefix of) the params PyTree,
+      or a Callable that returns such a pytree given the params/updates.
+      The leaves should be booleans, `True` for leaves/subtrees you want to
+      apply the transformation to, and `False` for those you want to skip.
 
   Returns:
     the corresponding `GradientTransformation`.
   """
   return combine.chain(
       transform.scale_by_adam(b1=b1, b2=b2, eps=eps, eps_root=eps_root),
-      transform.additive_weight_decay(weight_decay),
+      transform.add_decayed_weights(weight_decay, mask),
       _scale_by_learning_rate(learning_rate),
   )
 
 
 def fromage(
     learning_rate: float,
-    min_norm: float = 1e-6) -> GradientTransformation:
+    min_norm: float = 1e-6
+) -> base.GradientTransformation:
+  """The Frobenius matched gradient descent (Fromage) optimiser.
+
+  Fromage is a learning algorithm that does not require learning rate tuning.
+  The optimiser is based on modelling neural network gradients via deep relative
+  trust (a distance function on deep neural networks). Fromage is similar to the
+  LARS optimiser and can work on a range of standard neural network benchmarks,
+  such as natural language Transformers and generative adversarial networks.
+
+  References:
+    Bernstein et al, 2020: https://arxiv.org/abs/2002.03432
+
+  Args:
+    learning_rate: this is a fixed global scaling factor.
+    min_norm: a minimum value that the norm of the gradient updates and the
+    norm of the layer parameters can be clipped to to avoid dividing by zero
+    when computing the trust ratio (as in the LARS paper).
+
+  Returns:
+    the corresponding `GradientTransformation`.
+  """
   mult = 1 / jnp.sqrt(1 + learning_rate ** 2)
   return combine.chain(
       transform.scale_by_trust_ratio(min_norm),
@@ -165,16 +216,95 @@ def fromage(
   )
 
 
+def lars(
+    learning_rate: ScalarOrSchedule,
+    weight_decay: float = 0.,
+    weight_decay_mask: MaskOrFn = True,
+    trust_coefficient: float = 0.001,
+    eps: float = 0.,
+    trust_ratio_mask: MaskOrFn = True,
+    momentum: float = 0.9,
+    nesterov: bool = False,
+) -> base.GradientTransformation:
+  """The LARS optimiser.
+
+  LAMB is a layer-wise adaptive optimiser introduced to help scale SGD to
+  larger batch sizes. LARS later inspired the LAMB optimiser.
+
+  References:
+    You et al, 2017: https://arxiv.org/abs/1708.03888
+
+  Args:
+    learning_rate: this is a fixed global scaling factor.
+    weight_decay (default `0.`): strength of the weight decay regularization.
+    weight_decay_mask: a tree with same structure as (or a prefix of) the params
+      PyTree, or a Callable that returns such a pytree given the params/updates.
+      The leaves should be booleans, `True` for leaves/subtrees you want to
+      apply the transformation to, and `False` for those you want to skip.
+    trust_coefficient: a multiplier for the trust ratio.
+    eps: optional additive constant in the trust ratio denominator.
+    trust_ratio_mask: a tree with same structure as (or a prefix of) the params
+      PyTree, or a Callable that returns such a pytree given the params/updates.
+      The leaves should be booleans, `True` for leaves/subtrees you want to
+      apply the transformation to, and `False` for those you want to skip.
+    momentum: the decay rate for momentum.
+    nesterov: whether to use Nesterov momentum.
+
+  Returns:
+    the corresponding `GradientTransformation`.
+  """
+  return combine.chain(
+      transform.add_decayed_weights(weight_decay, mask=weight_decay_mask),
+      wrappers.masked(
+          inner=transform.scale_by_trust_ratio(
+              trust_coefficient=trust_coefficient, eps=eps),
+          mask=trust_ratio_mask),
+      _scale_by_learning_rate(learning_rate),
+      transform.trace(decay=momentum, nesterov=nesterov),
+  )
+
+
 def lamb(
     learning_rate: ScalarOrSchedule,
     b1: float = 0.9,
     b2: float = 0.999,
     eps: float = 1e-6,
     eps_root: float = 0.0,
-    weight_decay: float = 0.) -> GradientTransformation:
+    weight_decay: float = 0.,
+    mask: Optional[Union[Any, Callable[[base.Params], Any]]] = None,
+) -> base.GradientTransformation:
+  """The LAMB optimiser.
+
+  LAMB is a general purpose layer-wise adaptive large batch optimiser designed
+  to provide consistent training performance across a wide range of tasks,
+  including those that use attention-based models (such as Transformers) and
+  ResNet-50. The optimiser is able to work with small and large batch sizes.
+  LAMB was inspired by the LARS learning algorithm.
+
+  References:
+    You et al, 2019: https://arxiv.org/abs/1904.00962
+
+  Args:
+    learning_rate: this is a fixed global scaling factor.
+    b1: the exponential decay rate to track the first moment of past gradients.
+    b2: the exponential decay rate to track the second moment of past gradients.
+    eps: a small constant applied to denominator outside of the square root
+      (as in the Adam paper) to avoid dividing by zero when rescaling.
+    eps_root: (default `0.0`), a small constant applied to denominator inside
+      the square root (as in RMSProp), to avoid dividing by zero when rescaling.
+      This is needed for instance when computing (meta-)gradients through Adam.
+    weight_decay (default `0.`): strength of the weight decay regularization.
+    mask: a tree with same structure as (or a prefix of) the params PyTree,
+      or a Callable that returns such a pytree given the params/updates.
+      The leaves should be booleans, `True` for leaves/subtrees you want to
+      apply the transformation to, and `False` for those you want to skip.
+
+  Returns:
+    the corresponding `GradientTransformation`.
+  """
   return combine.chain(
       transform.scale_by_adam(b1=b1, b2=b2, eps=eps, eps_root=eps_root),
-      transform.add_decayed_weights(weight_decay),
+      transform.add_decayed_weights(weight_decay=weight_decay, mask=mask),
       transform.scale_by_trust_ratio(),
       _scale_by_learning_rate(learning_rate),
   )
@@ -184,14 +314,15 @@ def noisy_sgd(
     learning_rate: ScalarOrSchedule,
     eta: float = 0.01,
     gamma: float = 0.55,
-    seed: int = 0) -> GradientTransformation:
+    seed: int = 0
+) -> base.GradientTransformation:
   r"""A variant of SGD with added noise.
 
   It has been found that adding noise to the gradients can improve
   both the training error and the generalisation error in very deep networks.
 
   References:
-    [Neelakantan et al, 2014](https://arxiv.org/abs/1511.06807)
+    Neelakantan et al, 2014: https://arxiv.org/abs/1511.06807
 
   Args:
     learning_rate: this is a fixed global scaling factor.
@@ -215,7 +346,8 @@ def radam(
     b2: float = 0.999,
     eps: float = 1e-8,
     eps_root: float = 0.0,
-    threshold: float = 5.0) -> GradientTransformation:
+    threshold: float = 5.0
+) -> base.GradientTransformation:
   """The Rectified Adam optimiser.
 
   The adaptive learning rate in Adam has undesirably large variance in early
@@ -224,7 +356,7 @@ def radam(
   by analytically reducing the large variance.
 
   References:
-    [Kingma et al, 2014](https://arxiv.org/abs/1412.6980)
+    Kingma et al, 2014: https://arxiv.org/abs/1412.6980
 
   Args:
     learning_rate: this is a fixed global scaling factor.
@@ -255,7 +387,7 @@ def rmsprop(
     centered: bool = False,
     momentum: Optional[float] = None,
     nesterov: bool = False
-) -> GradientTransformation:
+) -> base.GradientTransformation:
   """A flexible RMSProp optimiser.
 
   RMSProp is an SGD variant with learning rate adaptation. The `learning_rate`
@@ -265,9 +397,9 @@ def rmsprop(
   optimiser that can be used to switch between several of these variants.
 
   References:
-    [Tieleman and Hinton, 2012](
-        www.cs.toronto.edu/~tijmen/csc321/slides/lecture_slides_lec6.pdf)
-    [Graves, 2013](https://arxiv.org/abs/1308.0850)
+    Tieleman and Hinton, 2012:
+        www.cs.toronto.edu/~tijmen/csc321/slides/lecture_slides_lec6.pdf
+    Graves, 2013: https://arxiv.org/abs/1308.0850
 
   Args:
     learning_rate: this is a fixed global scaling factor.
@@ -291,21 +423,22 @@ def rmsprop(
             decay=decay, eps=eps, initial_scale=initial_scale),
         _scale_by_learning_rate(learning_rate),
         (transform.trace(decay=momentum, nesterov=nesterov)
-         if momentum is not None else transform.identity())
+         if momentum is not None else base.identity())
     )
   return combine.chain(
       transform.scale_by_rms(
           decay=decay, eps=eps, initial_scale=initial_scale),
       _scale_by_learning_rate(learning_rate),
       (transform.trace(decay=momentum, nesterov=nesterov)
-       if momentum is not None else transform.identity())
+       if momentum is not None else base.identity())
   )
 
 
 def sgd(
     learning_rate: ScalarOrSchedule,
     momentum: Optional[float] = None,
-    nesterov: bool = False) -> GradientTransformation:
+    nesterov: bool = False
+) -> base.GradientTransformation:
   """A canonical Stochastic Gradient Descent optimiser.
 
   This implements stochastic gradient descent. It also includes support for
@@ -313,7 +446,7 @@ def sgd(
   using stochastic gradient descent to train deep neural networks.
 
   References:
-    [Sutskever et al, 2013](http://proceedings.mlr.press/v28/sutskever13.pdf)
+    Sutskever et al, 2013: http://proceedings.mlr.press/v28/sutskever13.pdf
 
   Args:
     learning_rate: this is a fixed global scaling factor.
@@ -326,16 +459,70 @@ def sgd(
   """
   return combine.chain(
       (transform.trace(decay=momentum, nesterov=nesterov)
-       if momentum is not None else transform.identity()),
+       if momentum is not None else base.identity()),
       _scale_by_learning_rate(learning_rate)
   )
 
 
+def sm3(
+    learning_rate: float,
+    momentum: float = 0.9
+) -> base.GradientTransformation:
+  """The SM3 optimiser.
+
+  SM3 (Square-root of Minima of Sums of Maxima of Squared-gradients Method) is a
+  memory-efficient adaptive optimiser designed to decrease memory overhead when
+  training very large models, such as the Transformer for machine translation,
+  BERT for language modelling, and AmoebaNet-D for image classification. SM3: 1)
+  applies to tensors of arbitrary dimensions and any predefined cover of the
+  parameters; 2) adapts the learning rates in an adaptive and data-driven manner
+  (like Adagrad and unlike Adafactor); and 3) comes with rigorous convergence
+  guarantees in stochastic convex optimization settings.
+
+  References:
+    Anil et al, 2019: https://arxiv.org/abs/1901.11150
+
+  Args:
+    learning_rate: this is a fixed global scaling factor.
+    momentum: the `decay` rate used by the momentum term (when it is not set to
+      `None`, then momentum is not used at all).
+
+  Returns:
+    the corresponding `GradientTransformation`.
+  """
+  return combine.chain(
+      transform.scale_by_sm3(momentum),
+      transform.scale(-learning_rate),
+  )
+
+
 def yogi(
     learning_rate: ScalarOrSchedule,
     b1: float = 0.9,
     b2: float = 0.999,
-    eps: float = 1e-3) -> GradientTransformation:
+    eps: float = 1e-3
+) -> base.GradientTransformation:
+  """The Yogi optimiser.
+
+  Yogi is an adaptive optimiser, which provides control in tuning the effective
+  learning rate to prevent it from increasing. By doing so, it focuses on
+  addressing the issues of convergence and generalisation in exponential moving
+  average-based adaptive methods (such as Adam and RMSprop). Yogi is a
+  modification of Adam and uses the same parameters.
+
+  References:
+    Zaheer et al, 2020: http://www.sanjivk.com/yogi_nips2018.pdf
+
+  Args:
+    learning_rate: this is a fixed global scaling factor.
+    b1: the exponential decay rate to track the first moment of past gradients.
+    b2: the exponential decay rate to track the second moment of past gradients.
+    eps: a small constant applied to denominator outside of the square root
+      (as in the Adam paper) to avoid dividing by zero when rescaling.
+
+  Returns:
+    the corresponding `GradientTransformation`.
+  """
   return combine.chain(
       transform.scale_by_yogi(b1=b1, b2=b2, eps=eps),
       _scale_by_learning_rate(learning_rate),
@@ -348,7 +535,8 @@ def dpsgd(
     noise_multiplier: float,
     seed: int,
     momentum: Optional[float] = None,
-    nesterov: bool = False) -> GradientTransformation:
+    nesterov: bool = False
+) -> base.GradientTransformation:
   """The DPSGD optimiser.
 
   Differential privacy is a standard for privacy guarantees of algorithms
@@ -361,7 +549,7 @@ def dpsgd(
   gradients as input (which are easy to obtain in JAX using `jax.vmap`).
 
   References:
-    [Abadi et al, 2016](https://arxiv.org/abs/1607.00133)
+    Abadi et al, 2016: https://arxiv.org/abs/1607.00133
 
   Args:
     learning_rate: this is a fixed global scaling factor.
@@ -381,6 +569,6 @@ def dpsgd(
           noise_multiplier=noise_multiplier,
           seed=seed),
       (transform.trace(decay=momentum, nesterov=nesterov)
-       if momentum is not None else transform.identity()),
+       if momentum is not None else base.identity()),
       _scale_by_learning_rate(learning_rate)
   )
