@@ -1,4 +1,3 @@
-# Lint as: python3
 # Copyright 2019 DeepMind Technologies Limited. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -38,6 +37,7 @@ class AliasTest(chex.TestCase):
       ('fromage', lambda: alias.fromage(1e-2)),
       ('adabelief', lambda: alias.adabelief(1e-1)),
       ('radam', lambda: alias.radam(1e-1)),
+      ('sm3', lambda: alias.sm3(1.0)),
       ('yogi', lambda: alias.yogi(1.0)),
       ('dpsgd', lambda: alias.dpsgd(1e-2, 10.0, 0.001, 0))
   )
@@ -76,6 +76,7 @@ class AliasTest(chex.TestCase):
       ('fromage', lambda: alias.fromage(5e-3)),
       ('adabelief', lambda: alias.adabelief(1e-1)),
       ('radam', lambda: alias.radam(1e-3)),
+      ('sm3', lambda: alias.sm3(1.0)),
       ('yogi', lambda: alias.yogi(1.0)),
       ('dpsgd', lambda: alias.dpsgd(2e-3, 10., 0.001, 0, 0.2))
   )
