@@ -1,4 +1,3 @@
-# Lint as: python3
 # Copyright 2019 DeepMind Technologies Limited. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -22,20 +21,20 @@ parameters or the exploration factor used to select actions.
 
 import functools
 import inspect
-from typing import Callable, Dict, Union, Optional, Iterable
+from typing import Callable, Dict, Union, Optional, Iterable, Sequence
 
 from absl import logging
 import chex
 import jax
 import jax.numpy as jnp
 
-from optax._src import transform
-from optax._src import utils
-
-Schedule = Callable[[chex.Numeric], chex.Numeric]
+from optax._src import base
+from optax._src import numerics
 
 
-def constant_schedule(value: Union[float, int]) -> Schedule:
+def constant_schedule(
+    value: Union[float, int]
+) -> base.Schedule:
   """Constructs a constant schedule.
 
   Args:
@@ -52,7 +51,8 @@ def polynomial_schedule(
     end_value: chex.Scalar,
     power: chex.Scalar,
     transition_steps: int,
-    transition_begin: int = 0) -> Schedule:
+    transition_begin: int = 0
+) -> base.Schedule:
   """Constructs a schedule with polynomial transition from init to end value.
 
   Args:
@@ -89,9 +89,22 @@ def polynomial_schedule(
   return schedule
 
 
+# Alias polynomial schedule to linear schedule for convenience.
+def linear_schedule(
+    init_value: chex.Scalar,
+    end_value: chex.Scalar,
+    transition_steps: int,
+    transition_begin: int = 0
+) -> base.Schedule:
+  return polynomial_schedule(
+      init_value=init_value, end_value=end_value, power=1,
+      transition_steps=transition_steps, transition_begin=transition_begin)
+
+
 def piecewise_constant_schedule(
     init_value: float,
-    boundaries_and_scales: Optional[Dict[int, float]] = None) -> Schedule:
+    boundaries_and_scales: Optional[Dict[int, float]] = None
+) -> base.Schedule:
   """Returns a function which implements a piecewise constant schedule.
 
   Args:
@@ -126,7 +139,8 @@ def exponential_decay(
     decay_rate: float,
     transition_begin: int = 0,
     staircase: bool = False,
-    end_value: Optional[float] = None) -> Schedule:
+    end_value: Optional[float] = None
+) -> base.Schedule:
   """Constructs a schedule with either continuous or discrete exponential decay.
 
   This function applies an exponential decay function to a provided initial
@@ -177,12 +191,12 @@ def exponential_decay(
     clip_fn = jnp.maximum if decay_rate < 1.0 else jnp.minimum
 
   def schedule(count):
-    count -= transition_begin
-    p = count / transition_steps
+    decreased_count = count - transition_begin
+    p = decreased_count / transition_steps
     if staircase:
       p = jnp.floor(p)
     decayed_value = jnp.where(
-        count <= 0, init_value, init_value * jnp.power(decay_rate, p))
+        decreased_count <= 0, init_value, init_value * jnp.power(decay_rate, p))
     if end_value is not None:
       decayed_value = clip_fn(decayed_value, end_value)
     return decayed_value
@@ -193,7 +207,8 @@ def exponential_decay(
 def cosine_decay_schedule(
     init_value: float,
     decay_steps: int,
-    alpha: float = 0.0) -> Schedule:
+    alpha: float = 0.0
+) -> base.Schedule:
   """Returns a function which implements cosine learning rate decay.
 
   For more details see: https://arxiv.org/abs/1608.03983
@@ -231,7 +246,8 @@ def _cosine_interpolate(start: float, end: float, pct: float):
 def piecewise_interpolate_schedule(
     interpolate_type: str,
     init_value: float,
-    boundaries_and_scales: Optional[Dict[int, float]] = None) -> Schedule:
+    boundaries_and_scales: Optional[Dict[int, float]] = None
+) -> base.Schedule:
   """Returns a function which implements a piecewise interpolated schedule.
 
   Args:
@@ -280,7 +296,8 @@ def linear_onecycle_schedule(
     pct_start: float = 0.3,
     pct_final: float = 0.85,
     div_factor: float = 25.0,
-    final_div_factor: float = 1e4) -> Schedule:
+    final_div_factor: float = 1e4
+) -> base.Schedule:
   """Returns a function which implements the onecycle learning rate schedule.
 
   This function uses a linear annealing strategy.
@@ -320,7 +337,8 @@ def cosine_onecycle_schedule(
     peak_value: float,
     pct_start: float = 0.3,
     div_factor: float = 25.0,
-    final_div_factor: float = 1e4) -> Schedule:
+    final_div_factor: float = 1e4
+) -> base.Schedule:
   """Returns a function which implements the onecycle learning rate schedule.
 
   This function uses a cosine annealing strategy.
@@ -352,6 +370,126 @@ def cosine_onecycle_schedule(
        int(transition_steps): 1. / (div_factor * final_div_factor)})
 
 
+def join_schedules(schedules: Sequence[base.Schedule],
+                   boundaries: Sequence[int]) -> base.Schedule:
+  """Sequentially apply multiple schedules.
+
+  Args:
+    schedules: A list of callables (expected to be optax schedules). Each
+      schedule will receive a step count indicating the number of steps since
+      the previous boundary transition.
+    boundaries: A list of integers (of length one less than schedules) that
+      indicate when to transition between schedules.
+  Returns:
+    schedule: A function that maps step counts to values.
+  """
+  def schedule(step: jnp.DeviceArray) -> jnp.DeviceArray:
+    output = schedules[0](step)
+    for boundary, schedule in zip(boundaries, schedules[1:]):
+      output = jnp.where(step < boundary, output, schedule(step - boundary))
+    return output
+  return schedule
+
+
+def warmup_cosine_decay_schedule(
+    init_value: float,
+    peak_value: float,
+    warmup_steps: int,
+    decay_steps: int,
+    end_value: float = 0.0
+) -> base.Schedule:
+  """Linear warmup followed by cosine decay.
+
+  Args:
+    init_value: Initial value for the scalar to be annealed.
+    peak_value: Peak value for scalar to be annealed at end of warmup.
+    warmup_steps: Positive integer, the length of the linear warmup.
+    decay_steps: Positive integer, the total length of the schedule. Note that
+      this includes the warmup time, so the number of steps during which cosine
+      annealing is applied is `decay_steps - warmup_steps`.
+    end_value: End value of the scalar to be annealed.
+  Returns:
+    schedule: A function that maps step counts to values.
+  """
+  schedules = [
+      linear_schedule(
+          init_value=init_value,
+          end_value=peak_value,
+          transition_steps=warmup_steps),
+      cosine_decay_schedule(
+          init_value=peak_value,
+          decay_steps=decay_steps - warmup_steps,
+          alpha=end_value/peak_value)]
+  return join_schedules(schedules, [warmup_steps])
+
+
+def warmup_exponential_decay_schedule(
+    init_value: float,
+    peak_value: float,
+    warmup_steps: int,
+    transition_steps: int,
+    decay_rate: float,
+    transition_begin: int = 0,
+    staircase: bool = False,
+    end_value: Optional[float] = None
+) -> base.Schedule:
+  """Linear warmup followed by exponential decay.
+
+  Args:
+    init_value: Initial value for the scalar to be annealed.
+    peak_value: Peak value for scalar to be annealed at end of warmup.
+    warmup_steps: Positive integer, the length of the linear warmup.
+    transition_steps: must be positive. See the decay computation above.
+    decay_rate: must not be zero. The decay rate.
+    transition_begin: must be positive. After how many steps to start annealing
+      (before this many steps the scalar value is held fixed at `init_value`).
+    staircase: if `True`, decay the values at discrete intervals.
+    end_value: the value at which the exponential decay stops. When
+      `decay_rate` < 1, `end_value` is treated as a lower bound, otherwise as
+      an upper bound. Has no effect when `decay_rate` = 0.
+  Returns:
+    schedule: A function that maps step counts to values.
+  """
+  schedules = [
+      linear_schedule(
+          init_value=init_value,
+          end_value=peak_value,
+          transition_steps=warmup_steps),
+      exponential_decay(
+          init_value=peak_value,
+          transition_steps=transition_steps,
+          decay_rate=decay_rate,
+          transition_begin=transition_begin,
+          staircase=staircase,
+          end_value=end_value)]
+  return join_schedules(schedules, [warmup_steps])
+
+
+def sgdr_schedule(cosine_kwargs: Iterable[Dict[str, chex.Numeric]]
+                  ) -> base.Schedule:
+  """SGD with warm restarts, from Loschilov & Hutter (arXiv:1608.03983).
+
+  This learning rate schedule applies multiple joined cosine decay cycles.
+  For more details see: https://arxiv.org/abs/1608.03983
+
+  Args:
+    cosine_kwargs: An Iterable of dicts, where each element specifies the
+      arguments to pass to each cosine decay cycle. The `decay_steps` kwarg
+      will specify how long each cycle lasts for, and therefore when to
+      transition to the next cycle.
+  Returns:
+    schedule: A function that maps step counts to values.
+  """
+  boundaries = []
+  schedules = []
+  step = 0
+  for kwargs in cosine_kwargs:
+    schedules += [warmup_cosine_decay_schedule(**kwargs)]
+    boundaries += [step + kwargs['decay_steps']]
+    step += kwargs['decay_steps']
+  return join_schedules(schedules, boundaries[:-1])
+
+
 def _convert_floats(x, dtype):
   """Convert float-like inputs to dtype, rest pass through."""
   if jax.dtypes.scalar_type_of(x) == float:
@@ -359,17 +497,17 @@ def _convert_floats(x, dtype):
   return x
 
 
-class InjectHyperparamsState(transform.OptState):
+class InjectHyperparamsState(base.OptState):
   """Maintains inner transform state, hyperparameters, and step count."""
   count: jnp.ndarray  # shape=(), dtype=jnp.int32
   hyperparams: Dict[str, chex.Numeric]
-  inner_state: transform.OptState
+  inner_state: base.OptState
 
 
 def inject_hyperparams(
-    inner_factory: Callable[..., transform.GradientTransformation],
+    inner_factory: Callable[..., base.GradientTransformation],
     static_args: Union[str, Iterable[str]] = ()
-) -> Callable[..., transform.GradientTransformation]:
+) -> Callable[..., base.GradientTransformation]:
   """Wrapper that injects hyperparameters into the inner GradientTransformation.
 
   This wrapper allows you to pass schedules (i.e. a function that returns a
@@ -423,7 +561,7 @@ def inject_hyperparams(
         f'{set(inner_signature.parameters.keys())}')
 
   @functools.wraps(inner_factory)
-  def wrapped_transform(*args, **kwargs) -> transform.GradientTransformation:
+  def wrapped_transform(*args, **kwargs) -> base.GradientTransformation:
     bound_arguments = inner_signature.bind(*args, **kwargs)
     bound_arguments.apply_defaults()
 
@@ -452,7 +590,7 @@ def inject_hyperparams(
           count, hparams, inner_factory(**other_hps, **hparams).init(params))
 
     def update_fn(updates, state, params=None):
-      count_inc = utils.safe_int32_increment(state.count)
+      count_inc = numerics.safe_int32_increment(state.count)
       dtype = getattr(next(iter(jax.tree_leaves(updates)), None), 'dtype', None)
       hparams = {k: _convert_floats(v, dtype)
                  for k, v in state.hyperparams.items()}
@@ -464,6 +602,6 @@ def inject_hyperparams(
       return updates, InjectHyperparamsState(count_inc, hparams, inner_state)
       # pylint:enable=too-many-function-args
 
-    return transform.GradientTransformation(init_fn, update_fn)
+    return base.GradientTransformation(init_fn, update_fn)
 
   return wrapped_transform
