@@ -1,4 +1,3 @@
-# Lint as: python3
 # Copyright 2019 DeepMind Technologies Limited. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,224 +14,135 @@
 # ==============================================================================
 """Gradient transformations."""
 
-from typing import Any, Callable, NamedTuple, Optional, Sequence, Tuple, Union
+import functools
+from typing import Any, Callable, NamedTuple, Optional, Union
 
 import chex
 import jax
 import jax.numpy as jnp
-from optax._src import utils
 
+from optax._src import base
+from optax._src import clipping
+from optax._src import numerics
+from optax._src import utils
+from optax._src import wrappers
 
 # pylint:disable=no-value-for-parameter
-OptState = NamedTuple  # Transformation states are (possibly empty) namedtuples.
-Params = Any  # Parameters are arbitrary nests of `jnp.ndarrays`.
-Updates = Params  # Gradient updates are of the same type as parameters.
-
-# Function used to initialise the transformation's state.
-TransformInitFn = Callable[
-    [Params],
-    Union[OptState, Sequence[OptState]]]
-# Function used to apply a transformation.
-TransformUpdateFn = Callable[
-    [Updates, OptState, Optional[Params]],
-    Tuple[Updates, OptState]]
 
 
-class GradientTransformation(NamedTuple):
-  """Optax transformations consists of a function pair: (initialise, update)."""
-  init: TransformInitFn
-  update: TransformUpdateFn
-
-
-NO_PARAMS_MSG = (
-    'You are using a transformation that requires the current value of'
-    ' parameters, but you are not passing `params` when calling `update`.')
-
-
-class IdentityState(OptState):
-  """The `identity` transformation is stateless."""
-
-
-def identity() -> GradientTransformation:
-  """Stateless identity transformation that leaves input gradients untouched.
-
-  Returns:
-    An (init_fn, update_fn) tuple.
-  """
-
-  def init_fn(_):
-    return IdentityState()
-
-  def update_fn(updates, state, params=None):
-    del params
-    return updates, state
-
-  return GradientTransformation(init_fn, update_fn)
-
-
-class ClipState(OptState):
-  """The `clip` transformation is stateless."""
-
-
-def clip(max_delta) -> GradientTransformation:
-  """Clip updates element-wise, to be between -max_delta and +max_delta.
-
-  Args:
-    max_delta: the maximum absolute value for each element in the update.
-
-  Returns:
-    An (init_fn, update_fn) tuple.
-  """
-
-  def init_fn(_):
-    return ClipState()
-
-  def update_fn(updates, state, params=None):
-    del params
-    updates = jax.tree_map(
-        lambda g: jnp.clip(g, -max_delta, max_delta), updates)
-    return updates, state
-
-  return GradientTransformation(init_fn, update_fn)
-
-
-def global_norm(updates: Updates) -> Updates:
-  return jnp.sqrt(
-      sum([jnp.sum(jnp.square(x)) for x in jax.tree_leaves(updates)]))
-
-
-class ClipByGlobalNormState(OptState):
-  """The `clip_by_global_norm` transformation is stateless."""
-
-
-def clip_by_global_norm(max_norm) -> GradientTransformation:
-  """Clip updates using their global norm.
-
-  References:
-    [Pascanu et al, 2012](https://arxiv.org/abs/1211.5063)
-
-  Args:
-    max_norm: the maximum global norm for an update.
-
-  Returns:
-    An (init_fn, update_fn) tuple.
-  """
-
-  def init_fn(_):
-    return ClipByGlobalNormState()
-
-  def update_fn(updates, state, params=None):
-    del params
-    g_norm = global_norm(updates)
-    # TODO(b/163995078): revert back to the following (faster) implementation
-    # once analysed how it affects backprop through update (e.g. meta-gradients)
-    # g_norm = jnp.maximum(max_norm, g_norm)
-    # updates = jax.tree_map(lambda t: (t / g_norm) * max_norm, updates)
-    trigger = g_norm < max_norm
-    updates = jax.tree_map(
-        lambda t: jnp.where(trigger, t, (t / g_norm) * max_norm), updates)
-    return updates, state
-
-  return GradientTransformation(init_fn, update_fn)
-
-
-def unitwise_norm(x):
-  """Computes norms of each output unit separately."""
-  if len(jnp.squeeze(x).shape) <= 1:  # Scalars and vectors
-    axis = None
-    keepdims = False
-  # Note that this assumes parameters with a shape of length 3 are multihead
-  # linear parameters--if you wish to apply AGC to 1D convs, you may need
-  # to modify this line.
-  elif len(x.shape) in [2, 3]:  # Linear layers of shape IO or multihead linear
-    axis = 0
-    keepdims = True
-  elif len(x.shape) == 4:  # Conv kernels of shape HWIO
-    axis = [0, 1, 2,]
-    keepdims = True
-  else:
-    raise ValueError(f'Got a parameter with shape not in [1, 2, 3, 4]! {x}')
-  return jnp.sum(x ** 2, axis=axis, keepdims=keepdims) ** 0.5
-
-
-def unitwise_clip(g_norm, max_norm, grad):
-  """Applies gradient clipping unit-wise."""
-  trigger = g_norm < max_norm
-  # This little max(., 1e-6) is distinct from the normal eps and just prevents
-  # division by zero. It technically should be impossible to engage.
-  clipped_grad = grad * (max_norm / jnp.maximum(g_norm, 1e-6))
-  return jnp.where(trigger, grad, clipped_grad)
-
-
-def adaptive_grad_clip(clipping, eps=1e-3) -> GradientTransformation:
-  """Clip updates to be at most clipping * parameter_norm, unit-wise.
-
-  References:
-    [Brock, Smith, De, Simonyan 2021] High-Performance Large-Scale Image
-    Recognition Without Normalization. (https://arxiv.org/abs/2102.06171)
-
-  Args:
-    clipping: Maximum allowed ratio of update norm to parameter norm.
-    eps: epsilon term to prevent clipping of zero-initialized params.
-
-  Returns:
-    An (init_fn, update_fn) tuple.
-  """
-
-  def init_fn(_):
-    return ClipByGlobalNormState()
-
-  def update_fn(updates, state, params):
-    if params is None:
-      raise ValueError(NO_PARAMS_MSG)
-    g_norm = jax.tree_map(unitwise_norm, updates)
-    p_norm = jax.tree_map(unitwise_norm, params)
-    # Maximum allowable norm
-    max_norm = jax.tree_map(lambda x: clipping * jnp.maximum(x, eps), p_norm)
-    # If grad norm > clipping * param_norm, rescale
-    updates = jax.tree_multimap(unitwise_clip, g_norm, max_norm, updates)
-    return updates, state
-
-  return GradientTransformation(init_fn, update_fn)
-
-
-class TraceState(OptState):
+class TraceState(base.OptState):
   """Holds an aggregation of past updates."""
-  trace: Params
+  trace: base.Params
 
 
-def trace(decay: float, nesterov: bool) -> GradientTransformation:
+def trace(
+    decay: float,
+    nesterov: bool = False,
+    accumulator_dtype: Optional[Any] = None,
+) -> base.GradientTransformation:
   """Compute a trace of past updates.
 
+  Note: `trace` and `ema` have very similar but distinct updates;
+  `trace = decay * trace + t`, while `ema = decay * ema + (1-decay) * t`.
+  Both are frequently found in the optimisation literature.
+
   Args:
     decay: the decay rate for the tracing of past updates.
     nesterov: whether to use Nesterov momentum.
+    accumulator_dtype: optional `dtype` to used for the accumulator; if `None`
+      then the `dtype` is inferred from `params` and `updates`.
 
   Returns:
     An (init_fn, update_fn) tuple.
   """
 
+  accumulator_dtype = utils.canonicalize_dtype(accumulator_dtype)
+
   def init_fn(params):
-    return TraceState(trace=jax.tree_map(jnp.zeros_like, params))
+    return TraceState(
+        trace=jax.tree_map(
+            lambda t: jnp.zeros_like(t, dtype=accumulator_dtype), params))
 
   def update_fn(updates, state, params=None):
     del params
     f = lambda g, t: g + decay * t
-    update_trace = jax.tree_multimap(f, updates, state.trace)
+    new_trace = jax.tree_multimap(f, updates, state.trace)
     updates = (
-        jax.tree_multimap(f, updates, update_trace)
-        if nesterov else update_trace)
-    return updates, TraceState(trace=update_trace)
+        jax.tree_multimap(f, updates, new_trace) if nesterov else new_trace)
+    new_trace = utils.cast_tree(new_trace, accumulator_dtype)
+    return updates, TraceState(trace=new_trace)
 
-  return GradientTransformation(init_fn, update_fn)
+  return base.GradientTransformation(init_fn, update_fn)
 
 
-class ScaleByRssState(OptState):
+def _update_moment(updates, moments, decay, order):
+  """Compute the exponential moving average of the `order-th` moment."""
+  return jax.tree_multimap(
+      lambda g, t: (1 - decay) * (g ** order) + decay * t, updates, moments)
+
+
+def _bias_correction(moment, decay, count):
+  """Perform bias correction. This becomes a no-op as count goes to infinity."""
+  bias_correction = 1 - decay**count
+  return jax.tree_map(lambda t: t / bias_correction.astype(t.dtype), moment)
+
+
+class EmaState(base.OptState):
+  """Holds an exponential moving average of past updates."""
+  count: chex.Array  # shape=(), dtype=jnp.int32.
+  ema: base.Params
+
+
+def ema(
+    decay: float,
+    debias: bool = True,
+    accumulator_dtype: Optional[Any] = None
+) -> base.GradientTransformation:
+  """Compute an exponential moving average of past updates.
+
+  Note: `trace` and `ema` have very similar but distinct updates;
+  `ema = decay * ema + (1-decay) * t`, while `trace = decay * trace + t`.
+  Both are frequently found in the optimisation literature.
+
+  Args:
+    decay: the decay rate for the exponential moving average.
+    debias: whether to debias the transformed gradient.
+    accumulator_dtype: optional `dtype` to used for the accumulator; if `None`
+      then the `dtype` is inferred from `params` and `updates`.
+
+  Returns:
+    An (init_fn, update_fn) tuple.
+  """
+
+  accumulator_dtype = utils.canonicalize_dtype(accumulator_dtype)
+
+  def init_fn(params):
+    return EmaState(
+        count=jnp.zeros([], jnp.int32),
+        ema=jax.tree_map(
+            lambda t: jnp.zeros_like(t, dtype=accumulator_dtype), params))
+
+  def update_fn(updates, state, params=None):
+    del params
+    new_ema = _update_moment(updates, state.ema, decay, order=1)
+    count_inc = utils.safe_int32_increment(state.count)
+    if debias:
+      new_ema = _bias_correction(new_ema, decay, count_inc)
+    state_ema = utils.cast_tree(new_ema, accumulator_dtype)
+    return new_ema, EmaState(count=count_inc, ema=state_ema)
+
+  return base.GradientTransformation(init_fn, update_fn)
+
+
+class ScaleByRssState(base.OptState):
   """State holding the sum of gradient squares to date."""
-  sum_of_squares: Updates
+  sum_of_squares: base.Updates
 
 
-def scale_by_rss(initial_accumulator_value: float = 0.1, eps: float = 1e-7):
+def scale_by_rss(
+    initial_accumulator_value: float = 0.1,
+    eps: float = 1e-7
+) -> base.GradientTransformation:
   """Rescale updates by the root of the sum of all squared gradients to date.
 
   References:
@@ -262,21 +172,19 @@ def scale_by_rss(initial_accumulator_value: float = 0.1, eps: float = 1e-7):
         lambda scale, g: scale * g, inv_sqrt_g_square, updates)
     return updates, ScaleByRssState(sum_of_squares=sum_of_squares)
 
-  return GradientTransformation(init_fn, update_fn)
+  return base.GradientTransformation(init_fn, update_fn)
 
 
-class ScaleByRmsState(OptState):
+class ScaleByRmsState(base.OptState):
   """State for exponential root mean-squared (RMS)-normalized updates."""
-  nu: Updates
-
-
-def _update_moment(updates, moments, decay, order):
-  return jax.tree_multimap(
-      lambda g, t: (1 - decay) * (g ** order) + decay * t, updates, moments)
+  nu: base.Updates
 
 
 def scale_by_rms(
-    decay: float = 0.9, eps: float = 1e-8, initial_scale: float = 0.):
+    decay: float = 0.9,
+    eps: float = 1e-8,
+    initial_scale: float = 0.
+) -> base.GradientTransformation:
   """Rescale updates by the root of the exp. moving avg of the square.
 
   References:
@@ -303,18 +211,20 @@ def scale_by_rms(
         lambda g, n: g * jax.lax.rsqrt(n + eps), updates, nu)
     return updates, ScaleByRmsState(nu=nu)
 
-  return GradientTransformation(init_fn, update_fn)
+  return base.GradientTransformation(init_fn, update_fn)
 
 
-class ScaleByRStdDevState(OptState):
+class ScaleByRStdDevState(base.OptState):
   """State for centered exponential moving average of squares of updates."""
-  mu: Updates
-  nu: Updates
+  mu: base.Updates
+  nu: base.Updates
 
 
 def scale_by_stddev(
-    decay: float = 0.9, eps: float = 1e-8,
-    initial_scale: float = 0.) -> GradientTransformation:
+    decay: float = 0.9,
+    eps: float = 1e-8,
+    initial_scale: float = 0.
+) -> base.GradientTransformation:
   """Rescale updates by the root of the centered exp. moving average of squares.
 
   References:
@@ -344,30 +254,22 @@ def scale_by_stddev(
         nu)
     return updates, ScaleByRStdDevState(mu=mu, nu=nu)
 
-  return GradientTransformation(init_fn, update_fn)
+  return base.GradientTransformation(init_fn, update_fn)
 
 
-class ScaleByAdamState(OptState):
+class ScaleByAdamState(base.OptState):
   """State for the Adam algorithm."""
-  count: jnp.ndarray  # shape=(), dtype=jnp.int32.
-  mu: Updates
-  nu: Updates
+  count: chex.Array  # shape=(), dtype=jnp.int32.
+  mu: base.Updates
+  nu: base.Updates
 
 
-# TODO(b/183478923): remove legacy reference.
-_safe_int32_increment = utils.safe_int32_increment
-
-
-def _bias_correction(moment, decay, count):
-  """Perform bias correction. This becomes a no-op as count goes to infinity."""
-  bias_correction = 1 - decay**count
-  return jax.tree_map(lambda t: t / bias_correction.astype(t.dtype), moment)
-
-
-def scale_by_adam(b1: float = 0.9,
-                  b2: float = 0.999,
-                  eps: float = 1e-8,
-                  eps_root: float = 0.0) -> GradientTransformation:
+def scale_by_adam(
+    b1: float = 0.9,
+    b2: float = 0.999,
+    eps: float = 1e-8,
+    eps_root: float = 0.0
+) -> base.GradientTransformation:
   """Rescale updates according to the Adam algorithm.
 
   References:
@@ -393,21 +295,22 @@ def scale_by_adam(b1: float = 0.9,
     del params
     mu = _update_moment(updates, state.mu, b1, 1)
     nu = _update_moment(updates, state.nu, b2, 2)
-    count_inc = utils.safe_int32_increment(state.count)
+    count_inc = numerics.safe_int32_increment(state.count)
     mu_hat = _bias_correction(mu, b1, count_inc)
     nu_hat = _bias_correction(nu, b2, count_inc)
     updates = jax.tree_multimap(
         lambda m, v: m / (jnp.sqrt(v + eps_root) + eps), mu_hat, nu_hat)
     return updates, ScaleByAdamState(count=count_inc, mu=mu, nu=nu)
 
-  return GradientTransformation(init_fn, update_fn)
+  return base.GradientTransformation(init_fn, update_fn)
 
 
-class ScaleState(NamedTuple):
-  """The scale transformation is stateless."""
+ScaleState = base.EmptyState
 
 
-def scale(step_size: float) -> GradientTransformation:
+def scale(
+    step_size: float
+) -> base.GradientTransformation:
   """Scale updates by some fixed scalar `step_size`.
 
   Args:
@@ -425,19 +328,76 @@ def scale(step_size: float) -> GradientTransformation:
     updates = jax.tree_map(lambda g: step_size * g, updates)
     return updates, state
 
-  return GradientTransformation(init_fn, update_fn)
+  return base.GradientTransformation(init_fn, update_fn)
 
 
-class ScaleByBeliefState(OptState):
+def scale_by_param_block_norm(
+    min_scale: float = 1e-3
+) -> base.GradientTransformation:
+  """Scale updates for each param block by the norm of that block's parameters.
+
+  A `block` is here a weight vector (e.g. in a Linear layer) or a weight matrix
+  (e.g. in a convolutional layer) appearing as a leaf in the grads/param pytree.
+
+  Args:
+    min_scale: minimum scaling factor.
+
+  Returns:
+    An (init_fn, update_fn) tuple.
+  """
+
+  def init_fn(_):
+    return base.EmptyState()
+
+  def update_fn(updates, state, params):
+    updates = jax.tree_multimap(
+        lambda u, p: u * numerics.safe_norm(p, min_scale),
+        updates, params)
+    return updates, state
+
+  return base.GradientTransformation(init_fn, update_fn)
+
+
+def scale_by_param_block_rms(
+    min_scale: float = 1e-3
+) -> base.GradientTransformation:
+  """Scale updates by rms of the gradient for each param vector or matrix.
+
+  A `block` is here a weight vector (e.g. in a Linear layer) or a weight matrix
+  (e.g. in a convolutional layer) appearing as a leaf in the grads/param pytree.
+
+  Args:
+    min_scale: minimum scaling factor.
+
+  Returns:
+    An (init_fn, update_fn) tuple.
+  """
+
+  def init_fn(_):
+    return base.EmptyState()
+
+  def update_fn(updates, state, params):
+    updates = jax.tree_map(
+        lambda u, p: u * numerics.safe_root_mean_squares(p, min_scale),
+        updates, params)
+    return updates, state
+
+  return base.GradientTransformation(init_fn, update_fn)
+
+
+class ScaleByBeliefState(base.OptState):
   """State for the rescaling by AdaBelief algorithm."""
-  count: jnp.ndarray  # shape=(), dtype=jnp.int32.
-  mu: Updates
-  nu: Updates
+  count: chex.Array  # shape=(), dtype=jnp.int32.
+  mu: base.Updates
+  nu: base.Updates
 
 
 def scale_by_belief(
-    b1: float = 0.9, b2: float = 0.999,
-    eps: float = 0., eps_root: float = 1e-16) -> GradientTransformation:
+    b1: float = 0.9,
+    b2: float = 0.999,
+    eps: float = 0.,
+    eps_root: float = 1e-16
+) -> base.GradientTransformation:
   """Rescale updates according to the AdaBelief algorithm.
 
   References:
@@ -464,21 +424,24 @@ def scale_by_belief(
     mu = _update_moment(updates, state.mu, b1, 1)
     prediction_error = jax.tree_multimap(lambda g, m: g-m, updates, state.mu)
     nu = _update_moment(prediction_error, state.nu, b2, 2)
-    count_inc = utils.safe_int32_increment(state.count)
+    count_inc = numerics.safe_int32_increment(state.count)
     mu_hat = _bias_correction(mu, b1, count_inc)
     nu_hat = _bias_correction(nu, b2, count_inc)
     updates = jax.tree_multimap(
         lambda m, v: m / (jnp.sqrt(v + eps_root) + eps), mu_hat, nu_hat)
     return updates, ScaleByBeliefState(count=count_inc, mu=mu, nu=nu)
 
-  return GradientTransformation(init_fn, update_fn)
+  return base.GradientTransformation(init_fn, update_fn)
 
 
 def scale_by_yogi(
-    b1: float = 0.9, b2: float = 0.999,
-    eps: float = 1e-3, eps_root: float = 0.0,
-    initial_accumulator_value: float = 1e-6) -> GradientTransformation:
-  """Rescale updates according to the Adam algorithm.
+    b1: float = 0.9,
+    b2: float = 0.999,
+    eps: float = 1e-3,
+    eps_root: float = 0.0,
+    initial_accumulator_value: float = 1e-6
+) -> base.GradientTransformation:
+  """Rescale updates according to the Yogi algorithm.
 
   References:
     [Zaheer et al, 2018](https://papers.nips.cc/paper/2018/hash/90365351ccc7437a1309dc64e4db32a3-Abstract.html) #pylint:disable=line-too-long
@@ -508,21 +471,23 @@ def scale_by_yogi(
     signed_sq = jax.tree_multimap(
         lambda g, v: jnp.sign(v - g**2)*g**2, updates, state.nu)
     nu = _update_moment(signed_sq, state.nu, b2, 2)
-    count_inc = utils.safe_int32_increment(state.count)
+    count_inc = numerics.safe_int32_increment(state.count)
     mu_hat = _bias_correction(mu, b1, count_inc)
     nu_hat = _bias_correction(nu, b2, count_inc)
     updates = jax.tree_multimap(
         lambda m, v: m / (jnp.sqrt(v + eps_root) + eps), mu_hat, nu_hat)
     return updates, ScaleByAdamState(count=count_inc, mu=mu, nu=nu)
 
-  return GradientTransformation(init_fn, update_fn)
+  return base.GradientTransformation(init_fn, update_fn)
 
 
-def scale_by_radam(b1: float = 0.9,
-                   b2: float = 0.999,
-                   eps: float = 1e-8,
-                   eps_root: float = 0.0,
-                   threshold: float = 5.0) -> GradientTransformation:
+def scale_by_radam(
+    b1: float = 0.9,
+    b2: float = 0.999,
+    eps: float = 1e-8,
+    eps_root: float = 0.0,
+    threshold: float = 5.0
+) -> base.GradientTransformation:
   """Rescale updates according to the Rectified Adam algorithm.
 
   References:
@@ -559,7 +524,7 @@ def scale_by_radam(b1: float = 0.9,
     del params
     mu = _update_moment(updates, state.mu, b1, 1)
     nu = _update_moment(updates, state.nu, b2, 2)
-    count_inc = utils.safe_int32_increment(state.count)
+    count_inc = numerics.safe_int32_increment(state.count)
     b2t = b2**count_inc
     ro = ro_inf - 2 * count_inc * b2t / (1 - b2t)
     mu_hat = _bias_correction(mu, b1, count_inc)
@@ -569,18 +534,24 @@ def scale_by_radam(b1: float = 0.9,
         (ro, mu_hat, nu_hat))
     return updates, ScaleByAdamState(count=count_inc, mu=mu, nu=nu)
 
-  return GradientTransformation(init_fn, update_fn)
+  return base.GradientTransformation(init_fn, update_fn)
 
 
-class AddDecayedWeightsState(NamedTuple):
-  """The decay transformation is stateless."""
+AddDecayedWeightsState = base.EmptyState
 
 
-def add_decayed_weights(weight_decay: float = 0.0) -> GradientTransformation:
+def add_decayed_weights(
+    weight_decay: float = 0.0,
+    mask: Optional[Union[Any, Callable[[base.Params], Any]]] = None
+) -> base.GradientTransformation:
   """Add parameter scaled by `weight_decay`.
 
   Args:
     weight_decay: a scalar weight decay rate.
+    mask: a tree with same structure as (or a prefix of) the params PyTree,
+      or a Callable that returns such a pytree given the params/updates.
+      The leaves should be booleans, `True` for leaves/subtrees you want to
+      apply the transformation to, and `False` for those you want to skip.
 
   Returns:
     An (init_fn, update_fn) tuple.
@@ -591,25 +562,27 @@ def add_decayed_weights(weight_decay: float = 0.0) -> GradientTransformation:
 
   def update_fn(updates, state, params):
     if params is None:
-      raise ValueError(NO_PARAMS_MSG)
+      raise ValueError(base.NO_PARAMS_MSG)
     updates = jax.tree_multimap(
         lambda g, p: g + weight_decay * p, updates, params)
     return updates, state
 
-  return GradientTransformation(init_fn, update_fn)
+  # If mask is not `None`, apply mask to the gradient transformation.
+  # E.g. it is common to skip weight decay on bias units and batch stats.
+  if mask is not None:
+    return wrappers.masked(
+        base.GradientTransformation(init_fn, update_fn), mask)
+  return base.GradientTransformation(init_fn, update_fn)
 
 
-# TODO(b/180608630): Remove deprecated references.
-AdditiveWeightDecayState = AddDecayedWeightsState
-additive_weight_decay = add_decayed_weights
-
-
-class ScaleByScheduleState(OptState):
+class ScaleByScheduleState(base.OptState):
   """Maintains count for scale scheduling."""
-  count: jnp.ndarray  # shape=(), dtype=jnp.int32
+  count: chex.Array  # shape=(), dtype=jnp.int32
 
 
-def scale_by_schedule(step_size_fn: Callable[[chex.Numeric], chex.Numeric]):
+def scale_by_schedule(
+    step_size_fn: base.Schedule
+) -> base.GradientTransformation:
   """Scale updates using a custom schedule for the `step_size`.
 
   Args:
@@ -629,21 +602,25 @@ def scale_by_schedule(step_size_fn: Callable[[chex.Numeric], chex.Numeric]):
     updates = jax.tree_map(
         lambda g: jnp.array(step_size, dtype=g.dtype) * g, updates)
     return updates, ScaleByScheduleState(
-        count=utils.safe_int32_increment(state.count))
+        count=numerics.safe_int32_increment(state.count))
 
-  return GradientTransformation(init_fn, update_fn)
+  return base.GradientTransformation(init_fn, update_fn)
 
 
-class ScaleByFromageState(OptState):
+class ScaleByFromageState(base.OptState):
   """Maintains count for step-size scheduling."""
-  count: jnp.ndarray  # shape=(), dtype=jnp.int32
+  count: chex.Array  # shape=(), dtype=jnp.int32
 
 
 class ScaleByTrustRatioState(NamedTuple):
   """The scale and decay trust ratio transformation is stateless."""
 
 
-def scale_by_trust_ratio(min_norm: float = 0.0) -> GradientTransformation:
+def scale_by_trust_ratio(
+    min_norm: float = 0.0,
+    trust_coefficient: float = 1.,
+    eps: float = 0.,
+) -> base.GradientTransformation:
   """Scale updates by trust ratio`.
 
   References:
@@ -651,6 +628,8 @@ def scale_by_trust_ratio(min_norm: float = 0.0) -> GradientTransformation:
 
   Args:
     min_norm: minimum norm for params and gradient norms; by default is zero.
+    trust_coefficient: a multiplier for the trust ratio.
+    eps: additive constant added to the denominator for numerical stability.
 
   Returns:
     An (init_fn, update_fn) tuple.
@@ -661,14 +640,14 @@ def scale_by_trust_ratio(min_norm: float = 0.0) -> GradientTransformation:
 
   def update_fn(updates, state, params):
     if params is None:
-      raise ValueError(NO_PARAMS_MSG)
+      raise ValueError(base.NO_PARAMS_MSG)
 
     def _scale_update(update, param):
 
       # Clip norms to minimum value, by default no clipping.
-      param_norm = utils.safe_norm(param, min_norm)
-      update_norm = utils.safe_norm(update, min_norm)
-      trust_ratio = param_norm / update_norm
+      param_norm = numerics.safe_norm(param, min_norm)
+      update_norm = numerics.safe_norm(update, min_norm)
+      trust_ratio = trust_coefficient * param_norm / (update_norm + eps)
 
       # If no minimum norm clipping is used
       # Set trust_ratio to 1 in case where parameters would never be updated.
@@ -681,16 +660,20 @@ def scale_by_trust_ratio(min_norm: float = 0.0) -> GradientTransformation:
     updates = jax.tree_multimap(_scale_update, updates, params)
     return updates, state
 
-  return GradientTransformation(init_fn, update_fn)
+  return base.GradientTransformation(init_fn, update_fn)
 
 
-class AddNoiseState(OptState):
+class AddNoiseState(base.OptState):
   """State for adding gradient noise. Contains a count for annealing."""
-  count: jnp.ndarray
-  rng_key: jnp.ndarray
+  count: chex.Array
+  rng_key: chex.PRNGKey
 
 
-def add_noise(eta: float, gamma: float, seed: int) -> GradientTransformation:
+def add_noise(
+    eta: float,
+    gamma: float,
+    seed: int
+) -> base.GradientTransformation:
   """Add gradient noise.
 
   References:
@@ -713,7 +696,7 @@ def add_noise(eta: float, gamma: float, seed: int) -> GradientTransformation:
     del params
     num_vars = len(jax.tree_leaves(updates))
     treedef = jax.tree_structure(updates)
-    count_inc = utils.safe_int32_increment(state.count)
+    count_inc = numerics.safe_int32_increment(state.count)
     variance = eta / count_inc**gamma
     all_keys = jax.random.split(state.rng_key, num=num_vars + 1)
     noise = jax.tree_multimap(
@@ -724,16 +707,18 @@ def add_noise(eta: float, gamma: float, seed: int) -> GradientTransformation:
         updates, noise)
     return updates, AddNoiseState(count=count_inc, rng_key=all_keys[0])
 
-  return GradientTransformation(init_fn, update_fn)
+  return base.GradientTransformation(init_fn, update_fn)
 
 
-class ApplyEvery(OptState):
+class ApplyEvery(base.OptState):
   """Contains a counter and a gradient accumulator."""
-  count: jnp.ndarray
-  grad_acc: Updates
+  count: chex.Array
+  grad_acc: base.Updates
 
 
-def apply_every(k: int = 1) -> GradientTransformation:
+def apply_every(
+    k: int = 1
+) -> base.GradientTransformation:
   """Accumulate gradients and apply them every k steps.
 
   Note that if this transformation is part of a chain, the states of the other
@@ -761,10 +746,10 @@ def apply_every(k: int = 1) -> GradientTransformation:
         lambda g, ga: acc * ga + g, updates, state.grad_acc)
     emit = c == (k - 1)
     updates = jax.tree_map(lambda ga: emit * ga, grad_acc)
-    count_inc = utils.safe_int32_increment(state.count)
+    count_inc = numerics.safe_int32_increment(state.count)
     return updates, ApplyEvery(count=count_inc % k, grad_acc=grad_acc)
 
-  return GradientTransformation(init_fn, update_fn)
+  return base.GradientTransformation(init_fn, update_fn)
 
 
 def _subtract_mean(g):
@@ -774,11 +759,10 @@ def _subtract_mean(g):
     return g
 
 
-class CentralState(OptState):
-  """The `centralize` transformation is stateless."""
+CentralState = base.EmptyState
 
 
-def centralize() -> GradientTransformation:
+def centralize() -> base.GradientTransformation:
   """Centralize gradients.
 
   References:
@@ -796,77 +780,89 @@ def centralize() -> GradientTransformation:
     updates = jax.tree_map(_subtract_mean, updates)
     return updates, state
 
-  return GradientTransformation(init_fn, update_fn)
+  return base.GradientTransformation(init_fn, update_fn)
 
 
-class NonNegativeParamsState(OptState):
-  """The `keep_params_nonnegative` transformation is stateless."""
+class ScaleBySM3State(base.OptState):
+  """State for the SM3 algorithm."""
+  mu: base.Updates
+  nu: base.Updates
 
 
-def keep_params_nonnegative() -> GradientTransformation:
-  """Modifies the updates to keep parameters non-negative, i.e. >= 0.
+def scale_by_sm3(
+    b1: float = 0.9,
+    b2: float = 1.0,
+    eps: float = 1e-8
+) -> base.GradientTransformation:
+  """Scale updates by sm3`.
 
-  This transformation ensures that parameters after the update will be
-  larger than or equal to zero.
-  In a chain of transformations, this should be the last one.
+  References:
+    [Anil et. al 2019](https://arxiv.org/abs/1901.11150)
 
-  WARNING: the transformation expects input params to be non-negative.
-  When params is negative the transformed update will move them to 0.
+  Args:
+    b1: decay rate for the exponentially weighted average of grads.
+    b2: decay rate for the exponentially weighted average of squared grads.
+    eps: term added to the denominator to improve numerical stability.
 
   Returns:
     An (init_fn, update_fn) tuple.
   """
 
-  def init_fn(_):
-    return NonNegativeParamsState()
-
-  def update_fn(updates, state, params):
-    if params is None:
-      raise ValueError(NO_PARAMS_MSG)
-
-    updates = jax.tree_multimap(
-        lambda p, u: jnp.where((p + u) < 0., -p, u), params, updates)
-    return updates, state
-
-  return GradientTransformation(init_fn, update_fn)
-
-
-class ZeroNansState(OptState):
-  """Contains a tree.
-
-  The entry `found_nan` has the same tree structure as that of the parameters.
-  Each leaf is a single boolean which contains True iff a NaN was detected in
-  the corresponding parameter array at the last call to `update`.
-  """
-  found_nan: Any
-
-
-def zero_nans() -> GradientTransformation:
-  """A transformation which replaces NaNs with 0.
-
-  Zeroing values in gradients is guaranteed to produce a direction of
-  non-increasing loss.
-
-  The state of the transformation has the same tree structure as that of the
-  parameters. Each leaf is a single boolean which contains True iff a NaN was
-  detected in the corresponding parameter array at the last call to `update`.
-  This state is not used by the transformation internally, but lets users be
-  aware when NaNs have been zeroed out.
-
-  Returns:
-    A `GradientTransformation`.
-  """
+  def zeros_for_dim(p):
+    return [jnp.zeros([s]) for s in p.shape]
 
   def init_fn(params):
-    return ZeroNansState(
-        jax.tree_map(lambda p: jnp.array(False, dtype=jnp.bool_), params))
+    mu = jax.tree_map(zeros_for_dim, params)
+    nu = jax.tree_map(jnp.zeros_like, params)
+    return ScaleBySM3State(mu, nu)
 
-  def update_fn(updates, opt_state, params=None):
+  def _expanded_shape(shape, axis):
+    # Replaces a `shape` of [M, N, K] with 1 in all dimensions except for i.
+    # For eg: i = 1 returns [1, N, 1].
+    rank = len(shape)
+    return [1] * axis + [shape[axis]] + [1] * (rank - axis - 1)
+
+  def _new_accum(g, v):
+    coeffs = ((1.0 - b2) if b2 != 1.0 else 1.0, b2)
+    if g.ndim < 2:
+      return coeffs[0]*g**2 + coeffs[1]*v[0]
+    else:
+      return coeffs[0]*g**2 + coeffs[1]*functools.reduce(jnp.minimum, v)
+
+  def _new_mu(g, i):
+    if g.ndim < 2:
+      return g
+    else:
+      return jnp.max(g, axis=other_axes(i, g.ndim))
+
+  def other_axes(idx, ndim):
+    return list(range(idx)) + list(range(idx+1, ndim))
+
+  def update_fn(updates, state, params=None):
     del params
-    opt_state = ZeroNansState(
-        jax.tree_map(lambda p: jnp.any(jnp.isnan(p)), updates))
-    updates = jax.tree_map(
-        lambda p: jnp.where(jnp.isnan(p), jnp.zeros_like(p), p), updates)
-    return updates, opt_state
+    mu = jax.tree_multimap(
+        lambda g, v:  # pylint:disable=g-long-lambda
+        [jnp.reshape(v[i], _expanded_shape(g.shape, i)) for i in range(g.ndim)],
+        updates, state.mu)
+    accum = jax.tree_multimap(_new_accum, updates, mu)
+    accum_inv_sqrt = jax.tree_map(
+        lambda t: jnp.where(t > 0, jax.lax.rsqrt(t + eps), 0.0), accum)
+    up = jax.tree_multimap(lambda g, a: g*a, updates, accum_inv_sqrt)
+    nu = _update_moment(up, state.nu, b1, 1)
+    mu = jax.tree_map(
+        lambda g: [_new_mu(g, i) for i in range(g.ndim)], accum)
 
-  return GradientTransformation(init=init_fn, update=update_fn)
+    return nu, ScaleBySM3State(mu=mu, nu=nu)
+
+  return base.GradientTransformation(init_fn, update_fn)
+
+
+# TODO(b/183800387): remove legacy aliases.
+# These legacy aliases are here for checkpoint compatibility
+# To be removed once checkpoints have updated.
+_safe_int32_increment = numerics.safe_int32_increment
+safe_int32_increment = numerics.safe_int32_increment
+AdditiveWeightDecayState = AddDecayedWeightsState
+additive_weight_decay = add_decayed_weights
+ClipState = clipping.ClipState
+ClipByGlobalNormState = clipping.ClipByGlobalNormState
