@@ -1,4 +1,3 @@
-# Lint as: python3
 # Copyright 2019 DeepMind Technologies Limited. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,9 +14,8 @@
 # ==============================================================================
 """Transformation wrappers."""
 
-from typing import Any, Callable, NamedTuple, Tuple, Union
+from typing import Any, Callable, NamedTuple, Union
 
-from absl import logging
 import jax
 from jax import lax
 import jax.numpy as jnp
@@ -26,15 +24,15 @@ from jax.tree_util import tree_map
 from jax.tree_util import tree_unflatten
 import numpy as np
 
-from optax._src import transform
+from optax._src import base
 
 Array = jnp.ndarray
 PyTree = Any
 
 
 def flatten(
-    inner: transform.GradientTransformation
-) -> transform.GradientTransformation:
+    inner: base.GradientTransformation
+) -> base.GradientTransformation:
   """Flattens parameters and gradients for init and update of inner transform.
 
   This can reduce the overhead of performing many calculations on lots of small
@@ -81,7 +79,7 @@ def flatten(
     updates = _unflatten(updates, updates_flat)
     return updates, state
 
-  return transform.GradientTransformation(init_fn, update_fn)
+  return base.GradientTransformation(init_fn, update_fn)
 
 
 class ApplyIfFiniteState(NamedTuple):
@@ -104,9 +102,9 @@ class ApplyIfFiniteState(NamedTuple):
 
 
 def apply_if_finite(
-    inner: transform.GradientTransformation,
+    inner: base.GradientTransformation,
     max_consecutive_errors: int
-) -> transform.GradientTransformation:
+) -> base.GradientTransformation:
   """A function that wraps an optimiser to make it robust to a few NaNs or Infs.
 
   The purpose of this function is to prevent any optimisation to happen if the
@@ -155,7 +153,7 @@ def apply_if_finite(
         total_notfinite=jnp.logical_not(isfinite) + state.total_notfinite,
         inner_state=new_inner_state)
 
-  return transform.GradientTransformation(init=init, update=update)
+  return base.GradientTransformation(init=init, update=update)
 
 
 def _zeros_tree_like(inp_tree):
@@ -194,7 +192,7 @@ class MultiSteps:
   """
 
   def __init__(self,
-               opt: transform.GradientTransformation,
+               opt: base.GradientTransformation,
                every_k_schedule: Union[int, Callable[[Array], Array]],
                use_grad_mean: bool = True):
     """Initialiser.
@@ -266,173 +264,8 @@ class MultiSteps:
   def has_updated(self, state: MultiStepsState) -> Array:
     return jnp.logical_and(state.mini_step == 0, state.gradient_step > 0)
 
-  def gradient_transformation(self) -> transform.GradientTransformation:
-    return transform.GradientTransformation(init=self.init, update=self.update)
-
-
-# pylint:disable=no-value-for-parameter
-class LookaheadState(transform.OptState):
-  """State of the `GradientTransformation` returned by `lookahead`.
-
-  Attributes:
-    fast_state: Optimizer state of the fast optimizer.
-    steps_since_sync: Number of fast optimizer steps taken since slow and fast
-      parameters were synchronized.
-  """
-  fast_state: transform.OptState
-  steps_since_sync: jnp.ndarray
-
-
-class LookaheadParams(NamedTuple):
-  """Holds a pair of slow and fast parameters for the lookahead optimizer.
-
-  Gradients should always be calculated with the fast parameters. The slow
-  parameters should be used for testing and inference as they generalize better.
-  See the reference for a detailed discussion.
-
-  References:
-    [Zhang et al, 2019](https://arxiv.org/pdf/1907.08610v1.pdf)
-
-  Attributes:
-    fast: Fast parameters.
-    slow: Slow parameters.
-  """
-  fast: transform.Params
-  slow: transform.Params
-
-  @classmethod
-  def init_synced(cls, params: transform.Params) -> 'LookaheadParams':
-    """Initialize a pair of synchronized lookahead parameters."""
-    return cls(slow=params, fast=params)
-
-
-def lookahead(fast_optimizer: transform.GradientTransformation,
-              sync_period: int,
-              slow_step_size: float,
-              reset_state: bool = False) -> transform.GradientTransformation:
-  """Lookahead optimizer.
-
-  Performs steps with a fast optimizer and periodically updates a set of slow
-  parameters. Optionally resets the fast optimizer state after synchronization
-  by calling the init function of the fast optimizer.
-
-  Updates returned by the lookahead optimizer should not be modified before they
-  are applied, otherwise fast and slow parameters are not synchronized
-  correctly.
-
-  References:
-    [Zhang et al, 2019](https://arxiv.org/pdf/1907.08610v1.pdf)
-
-  Args:
-    fast_optimizer: The optimizer to use in the inner loop of lookahead.
-    sync_period: Number of fast optimizer steps to take before synchronizing
-      parameters. Must be >= 1.
-    slow_step_size: Step size of the slow parameter updates.
-    reset_state: Whether to reset the optimizer state of the fast opimizer after
-      each synchronization.
-
-  Returns:
-    A `GradientTransformation` with init and update functions. The updates
-    passed to the update function should be calculated using the fast lookahead
-    parameters only.
-  """
-  if sync_period < 1:
-    raise ValueError('Synchronization period must be >= 1.')
-
-  def init_fn(params: transform.Params) -> LookaheadState:
-    try:
-      fast_params = params.fast
-    except AttributeError:
-      # Allowing init_fn to be called with fast parameters reduces the
-      # modifications necessary to adapt code to use lookahead in some cases.
-      logging.warning(
-          '`params` has no attribute `fast`. Continuing by assuming that '
-          'only fast parameters were passed to lookahead init.')
-      fast_params = params
-
-    return LookaheadState(
-        fast_state=fast_optimizer.init(fast_params),
-        steps_since_sync=jnp.zeros(shape=(), dtype=jnp.int32))
-
-  def update_fn(
-      updates: transform.Updates, state: LookaheadState,
-      params: LookaheadParams) -> Tuple[LookaheadParams, LookaheadState]:
-    updates, fast_state = fast_optimizer.update(updates, state.fast_state,
-                                                params)
-
-    sync_next = (state.steps_since_sync == sync_period - 1)
-    updates = _lookahead_update(updates, sync_next, params, slow_step_size)
-    if reset_state:
-      # Jittable way of resetting the fast optimizer state if parameters will be
-      # synchronized after this update step.
-      initial_state = fast_optimizer.init(params.fast)
-      fast_state = jax.tree_multimap(
-          lambda current, init: (1 - sync_next) * current + sync_next * init,
-          fast_state, initial_state)
-
-    steps_since_sync = (state.steps_since_sync + 1) % sync_period
-    return updates, LookaheadState(fast_state, steps_since_sync)
-
-  return transform.GradientTransformation(init_fn, update_fn)
-
-
-def _lookahead_update(
-    updates: transform.Updates, sync_next: bool, params: LookaheadParams,
-    slow_step_size: float) -> transform.GradientTransformation:
-  """Returns the updates corresponding to one lookahead step.
-
-  References:
-    [Zhang et al, 2019](https://arxiv.org/pdf/1907.08610v1.pdf)
-
-  Args:
-    updates: Updates returned by the fast optimizer.
-    sync_next: Wether fast and slow parameters should be synchronized after the
-      fast optimizer step.
-    params: Current fast and slow parameters as `LookaheadParams` object.
-    slow_step_size: Step size of the slow optimizer.
-
-  Returns:
-    The updates for the lookahead parameters.
-  """
-  # In the paper, lookahead is presented as two nested loops. To write lookahead
-  # as optax wrapper, these loops have to be broken into successive updates.
-  # This leads to two types of update steps:
-  #
-  # Non-synchronization steps (sync_next == False):
-  # The updates returned by the fast optimizer are used for the fast parameters
-  # without change and the slow parameter updates are zero (i.e. fast_updates =
-  # updates, slow_updates = 0).
-  #
-  # Synchronisation step (sync_next == True):
-  # This consists of two substeps: a last fast optimizer step and the
-  # synchronization.
-  #   Substep 1 (last fast optimizer step):
-  #     last_fast_params = fast_params + updates
-  #   Substep 2 (synchronization):
-  #     new_slow_params = slow_params + slow_step_size * (
-  #                       last_fast_params - slow_params)
-  #     new_fast_params = new_slow_params
-  #
-  #   Merging into a single update step we get the update rules:
-  #     slow_updates = slow_step_size * (fast_params + updates - slow_params)
-  #     fast_updates = new_slow_params - fast_params = updates - (1 -
-  #       slow_step_size) * (fast_params + updates - slow_params)
-  #
-  # To make the equations jittable, the two types of steps are merged. Defining
-  # last_difference = fast_params + updates - slow_params, this yields the
-  # following equtions which are implemented below:
-  #   slow_updates = slow_step_size * sync_next * last_difference
-  #   fast_updates = updates - (
-  #                  1 - slow_step_size) * sync_next * last_difference
-  last_difference = jax.tree_multimap(lambda f, u, s: f + u - s, params.fast,
-                                      updates, params.slow)
-  slow_updates = jax.tree_map(lambda diff: slow_step_size * sync_next * diff,
-                              last_difference)
-  fast_updates = jax.tree_multimap(
-      lambda up, diff: up - sync_next * (1 - slow_step_size) * diff, updates,
-      last_difference)
-
-  return LookaheadParams(fast=fast_updates, slow=slow_updates)
+  def gradient_transformation(self) -> base.GradientTransformation:
+    return base.GradientTransformation(init=self.init, update=self.update)
 
 
 class MaskedState(NamedTuple):
@@ -441,9 +274,9 @@ class MaskedState(NamedTuple):
 
 
 def masked(
-    inner: transform.GradientTransformation,
-    mask: Union[PyTree, Callable[[transform.Params], PyTree]]
-) -> transform.GradientTransformation:
+    inner: base.GradientTransformation,
+    mask: Union[PyTree, Callable[[base.Params], PyTree]]
+) -> base.GradientTransformation:
   """Mask updates so only a subset of them are computed.
 
   For example, it is common to skip weight decay for BatchNorm scale and all
@@ -468,11 +301,10 @@ def masked(
 
   Args:
     inner: Inner transformation to mask.
-    mask: A PyTree with the same structure as (or is a prefix of) the
-    parameters PyTree, or a Callable that returns such a pytree given the
-    parameters/updates. The leaves should be booleans which are `True` for
-    leaves/subtrees you want to apply the transformation to, and `False` for
-    those you want to skip.
+    mask: a PyTree with same structure as (or a prefix of) the params PyTree,
+      or a Callable that returns such a pytree given the params/updates.
+      The leaves should be booleans, `True` for leaves/subtrees you want to
+      apply the transformation to, and `False` for those you want to skip.
 
   Returns:
     New GradientTransformation wrapping `inner`.
@@ -508,7 +340,7 @@ def masked(
     new_updates = treedef.unflatten(flat_updates)
     return new_updates, MaskedState(inner_state=new_inner_state)
 
-  return transform.GradientTransformation(init_fn, update_fn)
+  return base.GradientTransformation(init_fn, update_fn)
 
 
 class MaybeUpdateState(NamedTuple):
@@ -518,9 +350,9 @@ class MaybeUpdateState(NamedTuple):
 
 
 def maybe_update(
-    inner: transform.GradientTransformation,
+    inner: base.GradientTransformation,
     should_update_fn: Callable[[Array], Array]
-) -> transform.GradientTransformation:
+) -> base.GradientTransformation:
   """Calls the inner update function only at certain steps.
 
   Creates a transformation wrapper which counts the number of times the `update`
@@ -556,4 +388,5 @@ def maybe_update(
         should_update_fn(state.step), do_update, reject_update, operand=None)
     return updates, MaybeUpdateState(new_inner_state, state.step + 1)
 
-  return transform.GradientTransformation(init_fn, update_fn)
+  return base.GradientTransformation(init_fn, update_fn)
+
